
#ifndef __GOTOPROGS__INSTRUCTIONS_H
    #define __GOTOPROGS__INSTRUCTIONS_H


    #include <string>
    #include <map>
    #include <vector>


    class ExecutionContext
    {
        public:
            std::map<std::string, int> Variables;
            int InstructionPointer;
    };



    class GotoInstruction;



    class GotoProgram
    {
        public:
            GotoProgram();
            ~GotoProgram();
            std::map<std::string, int> JumpLabels;
            std::vector<GotoInstruction*> Instructions;
            void Execute(ExecutionContext* context);
    };



    class GotoInstruction
    {
        public:
            GotoInstruction();
            virtual ~GotoInstruction();
            virtual void Execute(GotoProgram* program, ExecutionContext* context) = 0;
    };



    class Jump : public GotoInstruction
    {
        public:
            std::string Label;

            Jump(std::string Label);
            ~Jump();
            virtual void Execute(GotoProgram* program, ExecutionContext* context);
    };



    class ConditionalJump : public Jump
    {
        public:
            std::string Variable;
            int CmpNumber;
            std::string Label;

            ConditionalJump(std::string Variable, int CmpNumber, std::string Label);
            ~ConditionalJump();
            virtual void Execute(GotoProgram* program, ExecutionContext* context);
    };



    class Stop : public GotoInstruction
    {
        public:
            Stop();
            ~Stop();
            virtual void Execute(GotoProgram* program, ExecutionContext* context);
    };



    class Assignment : public GotoInstruction
    {
        public:
            std::string DstVariable;
            std::string SrcVariable;
            int Added;

            Assignment(std::string DstVariable, std::string SrcVariable, int Added);
            ~Assignment();
            virtual void Execute(GotoProgram* program, ExecutionContext* context);
    };


#endif