%code requires {

  #define YYERROR_VERBOSE
  #include <iostream>
  #include "instructions.h"
  class GotoLexer; // forward declaration

} // Yacc Deklarationen

%defines
%define parser_class_name {GotoParser}
%parse-param { GotoLexer &lexer }
%parse-param { GotoProgram** result }
%lex-param { GotoLexer &lexer }

%union
{
    std::string* stringval;
    int* intval;
    GotoInstruction* instrval;
    GotoProgram* progval;
};

%skeleton "lalr1.cc"
%start S

%token COLON SEMICOLON IF EQ THEN GOTO STOP ASSIGN ADD SUB
%token <intval> NUM
%token <stringval> ID

%left EQ
%left ADD SUB

%code {
    static int yylex(yy::GotoParser::semantic_type* yylval, GotoLexer &scanner);
}

%% // Produktionen
S
    : program
    {
        *result = $<progval>1;
    }
    ;

program
    :
    {
        $<progval>$ = new GotoProgram();
    }

    | program instruction SEMICOLON
    {
        $<progval>$ = $<progval>1;
        $<progval>$->Instructions.push_back($<instrval>2);
    }

    | program ID COLON
    {
        $<progval>$ = $<progval>1;
        $<progval>$->JumpLabels[*$<stringval>2] = $<progval>$->Instructions.size();
    }
    ;

instruction:
    | ID ASSIGN ID ADD NUM
    {
        $<instrval>$ = new Assignment( *$<stringval>1, *$<stringval>3, *$<intval>5 );
        delete $<stringval>1;
        delete $<intval>5;
        delete $<stringval>3;
    }

    | ID ASSIGN ID SUB NUM
    {
        $<instrval>$ = new Assignment( *$<stringval>1, *$<stringval>3, (-1) * (*$<intval>5) );
        delete $<stringval>1;
        delete $<intval>5;
        delete $<stringval>3;
    }

    | GOTO ID
    {
        $<instrval>$ = new Jump( *$<stringval>2 );
        delete $<stringval>2;
    }

    | IF ID EQ NUM THEN GOTO ID
    {
        $<instrval>$ = new ConditionalJump( *$<stringval>2, *$<intval>4, *$<stringval>7 );
        delete $<stringval>2;
        delete $<intval>4;
        delete $<stringval>7;
    }

    | STOP
    {
        $<instrval>$ = new Stop();
    }
    ;

%%

#include "gotolexer.h"

void yy::GotoParser::error(const std::string& s)
{
    std::cerr << "SYNTACTIC ERROR (Line " << lexer.GetCurrentLineNumber() << "): " << s << std::endl;
}

static int yylex(yy::GotoParser::semantic_type* yylval, GotoLexer &lexer)
{
    return lexer.yylex(yylval);
}
